/* THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR REPRESENTATIONS,
 * EITHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING ANY IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY
 * OR COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. TI DISCLAIMS
 * ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET POSSESSION, AND NON-INFRINGEMENT
 * OF ANY THIRD PARTY INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR
 * YOUR USE OF THE PROGRAM.
 * IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, CONSEQUENTIAL OR
 * INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY THEORY OF LIABILITY AND WHETHER OR
 * NOT TI HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY
 * OUT OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM.  EXCLUDED
 * DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF REMOVAL OR REINSTALLATION,
 * COMPUTER TIME, LABOR COSTS, LOSS OF GOODWILL, LOSS OF PROFITS, LOSS OF
 * SAVINGS, OR LOSS OF USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S
 * AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF YOUR USE OF THE
 * PROGRAM EXCEED FIVE HUNDRED DOLLARS (U.S.$500).
 * Unless otherwise stated, the Program written and copyrighted by Texas
 * Instruments is distributed as "freeware".  You may, only under TI's copyright
 * in the Program, use and modify the Program without any charge or restriction.
 * You may distribute to third parties, provided that you transfer a copy of this
 * license to the third party and the third party agrees to these terms by its
 * first use of the Program. You must reproduce the copyright notice and any
 * other legend of ownership on each copy or partial copy, of the Program.
 * You acknowledge and agree that the Program contains copyrighted material,
 * trade secrets and other TI proprietary information and is protected by
 * copyright laws, international copyright treaties, and trade secret laws, as
 * well as other intellectual property laws.  To protect TI's rights in the
 * Program, you agree not to decompile, reverse engineer, disassemble or
 * otherwise translate any object code versions of the Program to a
 * human-readable form.  You agree that in no event will you alter, remove or
 * destroy any copyright notice included in the Program.  TI reserves all rights
 * not specifically granted under this license. Except as specifically provided
 * herein, nothing in this agreement shall be construed as conferring by
 * implication, estoppel, or otherwise, upon you, any license or other right
 * under any TI patents, copyrights or trade secrets.
 * You may not use the Program in non-TI devices. */


//******************************************************************************
//
//  adc.c
//  Functions for using the integrated AD converter
//
//  Matthias Ulmann, Design Services - EMEA Power Management
//  Texas Instruments, Inc.
//  September 2013
//  Built with Code Composer Studio Core Edition Version: 5.5.0.00077
//
//******************************************************************************


#include "main.h"



// Initialization of the analog input
void init_adc(void)
{
	// Setup of inputs
	// Set pins as inputs
	clear_bit(A0_PDIR, A0_BIT);

	// Disable pull ups
	clear_bit(A0_RES, A0_BIT);

	// Enable analog input for A0
	set_bit(ADC10AE0, A0_BIT);


	// Select VCC and GND as reference
	clear_bit(ADC10CTL0, SREF2);
	clear_bit(ADC10CTL0, SREF1);
	clear_bit(ADC10CTL0, SREF0);
	
	// Set sample-and-hold time to 16 ADC10CLKs
	set_bit(ADC10CTL0, ADC10SHT1);
	clear_bit(ADC10CTL0, ADC10SHT0);
	
	// Set the sample rate to 200 ksps
	clear_bit(ADC10CTL0, ADC10SR);
	
	// No reference output
	clear_bit(ADC10CTL0, REFOUT);
	
	// Reference always buffered
	clear_bit(ADC10CTL0, REFBURST);
	
	// Continuous sample and conversion
	set_bit(ADC10CTL0, MSC);
	
	// Set reference generator voltage to 1.5V
	clear_bit(ADC10CTL0, REF2_5V);
	
	// Switch reference generator off
	clear_bit(ADC10CTL0, REFON);
	
	// Switch ADC on
	set_bit(ADC10CTL0, ADC10ON);
	
	// Interrupt enable
	set_bit(ADC10CTL0, ADC10IE);

	// No interrupt
	clear_bit(ADC10CTL0, ADC10IE);
	
	// Enable conversion
	set_bit(ADC10CTL0, ENC);

	// Select A0
	clear_bit(ADC10CTL1, INCH3);
	clear_bit(ADC10CTL1, INCH2);
	clear_bit(ADC10CTL1, INCH1);
	clear_bit(ADC10CTL1, INCH0);
	
	// ADC10SC bit as sample-and-hold trigger source
	clear_bit(ADC10CTL1, SHS1);
	clear_bit(ADC10CTL1, SHS0);
	
	// ADC10 data as straight binary
	clear_bit(ADC10CTL1, ADC10DF);
	
	// Sample-and-hold signal not inverted
	clear_bit(ADC10CTL1, ISSH);
	
	// Set clock divider to 8 (2 MHz)
	set_bit(ADC10CTL1, ADC10DIV2);
	set_bit(ADC10CTL1, ADC10DIV1);
	set_bit(ADC10CTL1, ADC10DIV0);
	
	// MCLK as source for ADC
	set_bit(ADC10CTL1, ADC10SSEL1);
	clear_bit(ADC10CTL1, ADC10SSEL0);
	
	// Repeat single channel conversion
	set_bit(ADC10CTL1, CONSEQ1);
	clear_bit(ADC10CTL1, CONSEQ0);
	
	// Start conversion
	set_bit(ADC10CTL0, ADC10SC);
}



// Run the AD converter and read-out the potentiometer
// The return value is the filtered raw value from the ADC
void get_analog_input(void)
{
	// Variables used for reading the analog input
	static volatile unsigned int poti_old=0;

	
	// Calculate new value (Linear Shift-Invariant with a factor of 8)
	poti_adc = (3 * poti_old) + ADC10MEM;
	poti_adc = poti_adc >> 2;

	// Save result for next calculation
	poti_old = poti_adc;

	// As the potentiometer is connected in the opposite direction,
	// the sampled values has to be "inverted"
	// 0 -> 1023, 1023 -> 0
	poti_adc = 1023 - poti_adc;

	// Start conversion
	set_bit(ADC10CTL0, ADC10SC);
}
