/* THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR REPRESENTATIONS,
 * EITHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING ANY IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY
 * OR COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. TI DISCLAIMS
 * ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET POSSESSION, AND NON-INFRINGEMENT
 * OF ANY THIRD PARTY INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR
 * YOUR USE OF THE PROGRAM.
 * IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, CONSEQUENTIAL OR
 * INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY THEORY OF LIABILITY AND WHETHER OR
 * NOT TI HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY
 * OUT OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM.  EXCLUDED
 * DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF REMOVAL OR REINSTALLATION,
 * COMPUTER TIME, LABOR COSTS, LOSS OF GOODWILL, LOSS OF PROFITS, LOSS OF
 * SAVINGS, OR LOSS OF USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S
 * AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF YOUR USE OF THE
 * PROGRAM EXCEED FIVE HUNDRED DOLLARS (U.S.$500).
 * Unless otherwise stated, the Program written and copyrighted by Texas
 * Instruments is distributed as "freeware".  You may, only under TI's copyright
 * in the Program, use and modify the Program without any charge or restriction.
 * You may distribute to third parties, provided that you transfer a copy of this
 * license to the third party and the third party agrees to these terms by its
 * first use of the Program. You must reproduce the copyright notice and any
 * other legend of ownership on each copy or partial copy, of the Program.
 * You acknowledge and agree that the Program contains copyrighted material,
 * trade secrets and other TI proprietary information and is protected by
 * copyright laws, international copyright treaties, and trade secret laws, as
 * well as other intellectual property laws.  To protect TI's rights in the
 * Program, you agree not to decompile, reverse engineer, disassemble or
 * otherwise translate any object code versions of the Program to a
 * human-readable form.  You agree that in no event will you alter, remove or
 * destroy any copyright notice included in the Program.  TI reserves all rights
 * not specifically granted under this license. Except as specifically provided
 * herein, nothing in this agreement shall be construed as conferring by
 * implication, estoppel, or otherwise, upon you, any license or other right
 * under any TI patents, copyrights or trade secrets.
 * You may not use the Program in non-TI devices. */


//******************************************************************************
//
//  io.c
//  Read-out of the switches and I/O's
//
//  Matthias Ulmann, Design Services - EMEA Power Management
//  Texas Instruments, Inc.
//  September 2013
//  Built with Code Composer Studio Core Edition Version: 5.5.0.00077
//
//******************************************************************************


#include "main.h"



// Variables for reading and debouncing of the inputs/switches
static volatile unsigned int ct0=0, ct1=0, rpt=0, key_i=0;		// Used for debouncing
volatile unsigned int key_press=0;		// Key press detect
volatile unsigned int key_rpt=0;		// key long press and repeat
volatile unsigned int key_state=0;		// Debounced and inverted key state: Bit = 1: key pressed
volatile unsigned int rpt_active=0;
volatile unsigned int rpt_state=0;

// Variables to save the state of the inputs/switches
volatile unsigned int key_enable_short=0, key_prog_short=0;
volatile unsigned int key_continuous_short=0, key_trigger_short;
volatile unsigned int key_enable_rpt=0, key_prog_rpt=0;
volatile unsigned int key_continuous_rpt=0, key_trigger_rpt=0;



// Initialization of the outputs
void init_outputs(void)
{
	// Set pins as outputs
	set_bit(PSU_ENABLE_PDIR, PSU_ENABLE_BIT);
	set_bit(LED_PROG_1_PDIR, LED_PROG_1_BIT);
	set_bit(LED_PROG_2_PDIR, LED_PROG_2_BIT);
	set_bit(LED_PROG_3_PDIR, LED_PROG_3_BIT);
	set_bit(LED_SINGLE_PDIR, LED_SINGLE_BIT);
	set_bit(LED_CONTINUOUS_PDIR, LED_CONTINUOUS_BIT);
	set_bit(LED_PULSE_PDIR, LED_PULSE_BIT);
	set_bit(TRIGGER_OUT_PDIR, TRIGGER_OUT_BIT);

	// Disable Pull-Ups
	clear_bit(PSU_ENABLE_RES, PSU_ENABLE_BIT);
	clear_bit(LED_PROG_1_RES, LED_PROG_1_BIT);
	clear_bit(LED_PROG_2_RES, LED_PROG_2_BIT);
	clear_bit(LED_PROG_3_RES, LED_PROG_3_BIT);
	clear_bit(LED_SINGLE_RES, LED_SINGLE_BIT);
	clear_bit(LED_CONTINUOUS_RES, LED_CONTINUOUS_BIT);
	clear_bit(LED_PULSE_RES, LED_PULSE_BIT);
	clear_bit(TRIGGER_OUT_RES, TRIGGER_OUT_BIT);

	// Set I/O function
	clear_bit(PSU_ENABLE_PSEL, PSU_ENABLE_BIT);
	clear_bit(LED_PROG_1_PSEL, LED_PROG_1_BIT);
	clear_bit(LED_PROG_2_PSEL, LED_PROG_2_BIT);
	clear_bit(LED_PROG_3_PSEL, LED_PROG_3_BIT);
	clear_bit(LED_SINGLE_PSEL, LED_SINGLE_BIT);
	clear_bit(LED_CONTINUOUS_PSEL, LED_CONTINUOUS_BIT);
	clear_bit(LED_PULSE_PSEL, LED_PULSE_BIT);
	clear_bit(TRIGGER_OUT_PSEL, TRIGGER_OUT_BIT);

	// Set outputs: LEDs off (high), other outputs: low
	clear_bit(PSU_ENABLE_POUT, PSU_ENABLE_BIT);
	clear_bit(LED_PROG_1_POUT, LED_PROG_1_BIT);
	clear_bit(LED_PROG_2_POUT, LED_PROG_2_BIT);
	clear_bit(LED_PROG_3_POUT, LED_PROG_3_BIT);
	clear_bit(LED_SINGLE_POUT, LED_SINGLE_BIT);
	clear_bit(LED_CONTINUOUS_POUT, LED_CONTINUOUS_BIT);
	clear_bit(LED_PULSE_POUT, LED_PULSE_BIT);
	set_bit(TRIGGER_OUT_POUT, TRIGGER_OUT_BIT);
}



// Initialization of the switches
// The input register is low when the switch is pressed
void init_switches(void)
{
	// set pins as inputs
	clear_bit(S1_PDIR, S1_BIT);
	clear_bit(S2_PDIR, S2_BIT);
	clear_bit(S3_PDIR, S3_BIT);
	clear_bit(S4_PDIR, S4_BIT);
	
	// Disable Pull-ups
	clear_bit(S1_RES, S1_BIT);
	clear_bit(S2_RES, S2_BIT);
	clear_bit(S3_RES, S3_BIT);
	clear_bit(S4_RES, S4_BIT);
	
	// Set I/O function
	clear_bit(S1_PSEL, S1_BIT);
	clear_bit(S2_PSEL, S2_BIT);
	clear_bit(S3_PSEL, S3_BIT);
	clear_bit(S4_PSEL, S4_BIT);
}



// Check if a key has been pressed.
// Each pressed key is reported only once
unsigned int get_key_press( unsigned int key_mask )
{
	_DINT();					// read and clear atomic, interrupts off!
	key_mask &= key_press;		// read key(s)
	key_press ^= key_mask;		// clear key(s)
	_EINT();					// enable interrupts
	
	return key_mask;
}



// Check if a key has been pressed long enough such that the
// key repeat functionality kicks in. After a small setup delay
// the key is reported being pressed in subsequent calls
// to this function. This simulates the user repeatedly
// pressing and releasing the key.
unsigned int get_key_rpt( unsigned int key_mask )
{
	_DINT();					// read and clear atomic, interrupts off!
	
	key_mask &= key_rpt;		// read key(s)	
	
	key_rpt = key_rpt ^ key_mask;	// delete key(s)
 
	
	_EINT();					// enable interrupts
	
	return key_mask;
}



// Check if a key is pressed short.
unsigned int get_key_short( unsigned int key_mask )
{
	return get_key_press( ~key_state & key_mask );
}



// Read and debounce inputs/switches - has to be executed every 10ms.
void read_debounce_inputs(void)
{
	// Process switches/inputs
	key_i = key_state ^ SWITCH_PIN;		// Key changed?

	key_i = key_i & BIT_MASK;			// Throw away other bits
	key_state = key_state & BIT_MASK;	// Throw away other bits

	ct0 = ~( ct0 & key_i );				// Reset or count ct0
	ct0 = ct0 & BIT_MASK;				// Throw away other bits
	ct1 = ct0 ^ (ct1 & key_i);			// Reset or count ct1
	ct1 = ct1 & BIT_MASK;				// Throw away other bits
	key_i &= ct0 & ct1;					// Count until roll over ?
	key_state ^= key_i;					// Then toggle debounced state
	key_press |= key_state & key_i;		// 1->0: key press detect

	if( (key_state != BIT_MASK)	&& !(rpt_active))	// Check repeat function
	{
		rpt_state = ~key_state;						// Save key status
		rpt_state = rpt_state & BIT_MASK;			// Throw away other bits
		rpt = REPEAT_START;							// Start delay
		rpt_active = 1;
	}

	if(rpt_active) rpt--;

	if( (rpt == 0) && rpt_active)
	{
		key_rpt = rpt_state & (~key_state);		// Check if the key is still pressed
		key_rpt = key_rpt & BIT_MASK;			// Throw away other bits
		rpt_active = 0;							// Reset flag
	}
}



// Process inputs and set variables for program flow
void process_inputs(void)
{
	//*************************************************************
	// Process switches/inputs and set variables for program flow
	// BEGINNING
	//*************************************************************

	// Read switches/inputs and store status in variables
	if(get_key_short(SWITCH_ENABLE)) key_enable_short = 1;
	else key_enable_short = 0;

	if(get_key_short(SWITCH_PROGRAM)) key_prog_short = 1;
	else key_prog_short = 0;

	if(get_key_short(SWITCH_CONTINUOUS)) key_continuous_short = 1;
	else key_continuous_short = 0;

	if(get_key_short(SWITCH_TRIGGER_IN)) key_trigger_short = 1;
	else key_trigger_short = 0;


	// Enable/disabled power supply
	if(key_enable_short)
	{
		switch(status_enable)
		{
		case 0:	// Power supply is off and has to be switched on
			PSU_on;					// Switch power supply on
			status_enable = 1;		// Save state
			key_enable_short = 0;	// Clear flag
			break;

		case 1: // Power supply is on and has to be switched off
			PSU_off;				// Switch power supply off
			status_enable = 0;		// Save state
			key_enable_short = 0;	// Clear flag
			break;
		}
	}


	// Select program 1, 2 or 3
	if(key_prog_short)
	{
		switch(status_prog)
		{
		case 1:	// Program 1 is active, switch to program 2
			LED_PROG_1_off;			// Switch LED for program 1 off
			LED_PROG_2_on;			// Switch LED for program 2 on
			status_prog = 2;		// Save state
			set_default_voltage();	// Switch to default output voltage for this program
			key_prog_short = 0;		// Clear flag "key pressed"
			break;

		case 2: // Program 2 is active, switch to program 3
			LED_PROG_2_off;			// Switch LED for program 2 off
			LED_PROG_3_on;			// Switch LED for program 3 on
			status_prog = 3;		// Save state
			set_default_voltage();	// Switch to default output voltage for this program
			key_prog_short = 0;		// Clear flag "key pressed"
			break;

		case 3: // Program 3 is active, switch back to program 1
			LED_PROG_3_off;			// Switch LED for program 3 off
			LED_PROG_1_on;			// Switch LED for program 1 on
			status_prog = 1;		// Save state
			set_default_voltage();	// Switch to default output voltage for this program
			key_prog_short = 0;		// Clear flag "key pressed"
			break;
		}
	}


	// Set continuous or single shot mode
	if(key_continuous_short)
	{
		switch(status_continuous)
		{
		case 0:	// Single shot mode is active, switch to continuous mode
			LED_SINGLE_off;				// Switch single mode LED off
			LED_CONTINUOUS_on;			// Switch continuous mode LED on
			status_continuous = 1;		// Save state
			key_continuous_short = 0;	// Clear flag
			break;

		case 1:	// Continuous mode is active, switch to single shot mode
			LED_CONTINUOUS_off;			// Switch continuous mode LED off
			LED_SINGLE_on;				// Switch single mode LED on
			status_continuous = 0;		// Save state
			key_continuous_short = 0;	// Clear flag
			break;
		}
	}

	//*************************************************************
	// Process switches/inputs and set variables for program flow
	// END
	//*************************************************************



/*	State 0		Stop generating a single pulse or continuous pulse
 *
 * 	State 1		Start generating a single pulse
 *
 * 	State 2		A single pulse is generated at the moment
 * 				Is set by loop in main
 *
 * 	State 3		Start generating continuous pulses
 *
 * 	State 4		Continuous pulses are generated at the moment
 * 				Is set by loop in main
 */


	if(key_trigger_short)
	{
		// Switch between actions depending on the previous state
		switch(status_pulse)
		{
		// Previous: 	No pulses have to be generated
		// Now: 		Pulses have to be generated
		case 0:
			if(status_continuous == 0)	// Single shot mode is selected
			{
				status_pulse = 1;		// A single pulse has to be generated
			}
			else						// Continuous mode is selected
			{
				status_pulse = 3;		// Continuous pulses have to be generated
			}
			break;

		// Previous: 	Single pulses have to be generated
		// Now:			Stop generating single pulses
		case 1:
			status_pulse = 0;			// Stop generating a single pulse
			break;

		// Previous:	A single pulse is generated at the moment
		// Now:			Stop generating single pulses
		case 2:
			status_pulse = 0;			// Stop generating a single pulse
			break;

		// Previous:	Continuous pulses have to be generated
		// Now:			Stop generating continuous pulses
		case 3:
			status_pulse = 0;			// Stop generating continuous pulses
			break;

		// Previous:	Continuous pulses are generated at the moment
		// Now:			Stop generating continuous pulses
		case 4:
			status_pulse = 0;
			break;
		}
	}


	// Process analog input for potentiometer
	get_analog_input();
}
