/* THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR REPRESENTATIONS,
 * EITHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING ANY IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY
 * OR COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. TI DISCLAIMS
 * ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET POSSESSION, AND NON-INFRINGEMENT
 * OF ANY THIRD PARTY INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR
 * YOUR USE OF THE PROGRAM.
 * IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, CONSEQUENTIAL OR
 * INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY THEORY OF LIABILITY AND WHETHER OR
 * NOT TI HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY
 * OUT OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM.  EXCLUDED
 * DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF REMOVAL OR REINSTALLATION,
 * COMPUTER TIME, LABOR COSTS, LOSS OF GOODWILL, LOSS OF PROFITS, LOSS OF
 * SAVINGS, OR LOSS OF USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S
 * AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF YOUR USE OF THE
 * PROGRAM EXCEED FIVE HUNDRED DOLLARS (U.S.$500).
 * Unless otherwise stated, the Program written and copyrighted by Texas
 * Instruments is distributed as "freeware".  You may, only under TI's copyright
 * in the Program, use and modify the Program without any charge or restriction.
 * You may distribute to third parties, provided that you transfer a copy of this
 * license to the third party and the third party agrees to these terms by its
 * first use of the Program. You must reproduce the copyright notice and any
 * other legend of ownership on each copy or partial copy, of the Program.
 * You acknowledge and agree that the Program contains copyrighted material,
 * trade secrets and other TI proprietary information and is protected by
 * copyright laws, international copyright treaties, and trade secret laws, as
 * well as other intellectual property laws.  To protect TI's rights in the
 * Program, you agree not to decompile, reverse engineer, disassemble or
 * otherwise translate any object code versions of the Program to a
 * human-readable form.  You agree that in no event will you alter, remove or
 * destroy any copyright notice included in the Program.  TI reserves all rights
 * not specifically granted under this license. Except as specifically provided
 * herein, nothing in this agreement shall be construed as conferring by
 * implication, estoppel, or otherwise, upon you, any license or other right
 * under any TI patents, copyrights or trade secrets.
 * You may not use the Program in non-TI devices. */


//******************************************************************************
//
//  io.h
//  Read-out of the switches and I/O's
//
//  Matthias Ulmann, Design Services - EMEA Power Management
//  Texas Instruments, Inc.
//  September 2013
//  Built with Code Composer Studio Core Edition Version: 5.5.0.00077
//
//******************************************************************************


#ifndef IO_H
#define IO_H



// Global variables for program flow
extern volatile unsigned int status_enable;			// Power supply enabled/disabled LOCAL
extern volatile unsigned int status_continuous;		// Continuous or single shot mode  LOCAL
extern volatile unsigned int status_pulse;			// Pulse generation active or not  LOCAL
extern volatile unsigned int status_prog;			// Program 1, 2 or 3  GLOBAL



//******************************************************************************
// Definition of the outputs
// PSU_ENABLE, PROG_1, PROG_2, PROG_3,
// LED_SINGLE, LED_CONTINUOUS, LED_PULSE, TRIGGER_OUT
//******************************************************************************

// Bits for outputs
#define PSU_ENABLE_BIT		BIT4
#define LED_PROG_1_BIT		BIT3
#define LED_PROG_2_BIT		BIT7
#define LED_PROG_3_BIT		BIT6
#define LED_SINGLE_BIT		BIT6
#define LED_CONTINUOUS_BIT	BIT5
#define LED_PULSE_BIT		BIT4
#define TRIGGER_OUT_BIT		BIT7

// Function select register
#define PSU_ENABLE_PSEL		P2SEL
#define LED_PROG_1_PSEL		P2SEL
#define LED_PROG_2_PSEL		P3SEL
#define LED_PROG_3_PSEL		P3SEL
#define LED_SINGLE_PSEL		P4SEL
#define LED_CONTINUOUS_PSEL	P4SEL
#define LED_PULSE_PSEL		P4SEL
#define TRIGGER_OUT_PSEL	P4SEL

// Direction register
#define PSU_ENABLE_PDIR		P2DIR
#define LED_PROG_1_PDIR		P2DIR
#define LED_PROG_2_PDIR		P3DIR
#define LED_PROG_3_PDIR		P3DIR
#define LED_SINGLE_PDIR		P4DIR
#define LED_CONTINUOUS_PDIR	P4DIR
#define LED_PULSE_PDIR		P4DIR
#define TRIGGER_OUT_PDIR	P4DIR

// Output register
#define PSU_ENABLE_POUT		P2OUT
#define LED_PROG_1_POUT		P2OUT
#define LED_PROG_2_POUT		P3OUT
#define LED_PROG_3_POUT		P3OUT
#define LED_SINGLE_POUT		P4OUT
#define LED_CONTINUOUS_POUT	P4OUT
#define LED_PULSE_POUT		P4OUT
#define TRIGGER_OUT_POUT	P4OUT

// Input register
#define PSU_ENABLE_PIN		P2IN
#define LED_PROG_1_PIN		P2IN
#define LED_PROG_2_PIN		P3IN
#define LED_PROG_3_PIN		P3IN
#define LED_SINGLE_PIN		P4IN
#define LED_CONTINUOUS_PIN	P4IN
#define LED_PULSE_PIN		P4IN
#define TRIGGER_OUT_PIN		P4IN

// Pull/up/-down resistor register
#define PSU_ENABLE_RES		P2REN
#define LED_PROG_1_RES		P2REN
#define LED_PROG_2_RES		P3REN
#define LED_PROG_3_RES		P3REN
#define LED_SINGLE_RES		P4REN
#define LED_CONTINUOUS_RES	P4REN
#define LED_PULSE_RES		P4REN
#define TRIGGER_OUT_RES		P4REN



//******************************************************************************
// Switch outputs on/off
//******************************************************************************

// Switch PSU on/off
#define PSU_on				set_bit(PSU_ENABLE_POUT, PSU_ENABLE_BIT)
#define PSU_off				clear_bit(PSU_ENABLE_POUT, PSU_ENABLE_BIT)

// Switch PROG_1 on/off
#define LED_PROG_1_on		set_bit(LED_PROG_1_POUT, LED_PROG_1_BIT)
#define LED_PROG_1_off		clear_bit(LED_PROG_1_POUT, LED_PROG_1_BIT)

// Switch PROG_2 on/off
#define LED_PROG_2_on		set_bit(LED_PROG_2_POUT, LED_PROG_2_BIT)
#define LED_PROG_2_off		clear_bit(LED_PROG_2_POUT, LED_PROG_2_BIT)

// Switch PROG_3 on/off
#define LED_PROG_3_on		set_bit(LED_PROG_3_POUT, LED_PROG_3_BIT)
#define LED_PROG_3_off		clear_bit(LED_PROG_3_POUT, LED_PROG_3_BIT)

// Switch LED_SINGLE on/off
#define LED_SINGLE_on		set_bit(LED_SINGLE_POUT, LED_SINGLE_BIT)
#define LED_SINGLE_off		clear_bit(LED_SINGLE_POUT, LED_SINGLE_BIT)

// Switch LED_CONTINUOUS on/off
#define LED_CONTINUOUS_on	set_bit(LED_CONTINUOUS_POUT, LED_CONTINUOUS_BIT)
#define LED_CONTINUOUS_off	clear_bit(LED_CONTINUOUS_POUT, LED_CONTINUOUS_BIT)

// Switch LED_PULSE on/off
#define LED_PULSE_on		set_bit(LED_PULSE_POUT, LED_PULSE_BIT)
#define LED_PULSE_off		clear_bit(LED_PULSE_POUT, LED_PULSE_BIT)

// Switch TRIGGER_OUT on/off
#define TRIGGER_OUT_on		clear_bit(TRIGGER_OUT_POUT, TRIGGER_OUT_BIT)
#define TRIGGER_OUT_off		set_bit(TRIGGER_OUT_POUT, TRIGGER_OUT_BIT)



//******************************************************************************
// Definition of the switches/ inputs
// ENABLE, PROG, CONTINUOUS, TRIGGER_IN
//******************************************************************************

// Bits for switches S1, S2, S3, S4
#define S1_BIT			BIT0
#define S2_BIT			BIT1
#define S3_BIT			BIT2
#define S4_BIT			BIT3

// Function select register for S1, S2, S3, S4
#define S1_PSEL			P3SEL
#define S2_PSEL			P3SEL
#define S3_PSEL			P3SEL
#define S4_PSEL			P3SEL

// Direction register for S1, S2, S3, S4
#define S1_PDIR			P3DIR
#define S2_PDIR			P3DIR
#define S3_PDIR			P3DIR
#define S4_PDIR			P3DIR

// Output register for S1, S2, S3, S4
#define S1_POUT			P3OUT
#define S2_POUT			P3OUT
#define S3_POUT			P3OUT
#define S4_POUT			P3OUT

// Input register for S1, S2, S3, S4
#define S1_PIN			P3IN
#define S2_PIN			P3IN
#define S3_PIN			P3IN
#define S4_PIN			P3IN

// Pull/up/-down resistor register for S1, S2, S3, S4
#define S1_RES			P3REN
#define S2_RES			P3REN
#define S3_RES			P3REN
#define S4_RES			P3REN

// Definitions for switches
#define SWITCH_ENABLE		S1_BIT
#define SWITCH_PROGRAM		S2_BIT
#define SWITCH_CONTINUOUS	S3_BIT
#define SWITCH_TRIGGER_IN	S4_BIT

// Input port
#define SWITCH_PIN		P3IN

#define BIT_MASK		(S1_BIT | S2_BIT | S3_BIT | S4_BIT)
#define REPEAT_MASK		(S1_BIT | S2_BIT | S3_BIT | S4_BIT)

// define time for repeat function
#define REPEAT_START    100		// after 1000ms
#define REPEAT_NEXT     20		// every 200ms


// Initialization of the outputs
extern void init_outputs(void);

// Initialization of the switches
extern void init_switches(void);

// Check if a key has been pressed. Each pressed key is reported only once.
extern unsigned int get_key_press( unsigned int key_mask );

// Check if a key has been pressed long enough such that the
// key repeat functionality kicks in. After a small setup delay
// the key is reported being pressed in subsequent calls
// to this function. This simulates the user repeatedly
// pressing and releasing the key.
extern unsigned int get_key_rpt(unsigned int key_mask);

//Check if a key is pressed short.
extern unsigned int get_key_short(unsigned int key_mask);

// Read and debounce inputs/switches - has to be executed every 10ms.
extern void read_debounce_inputs(void);

// Process inputs and set variables for program flow
extern void process_inputs(void);



#endif /*IO_H*/
