/* THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR REPRESENTATIONS,
 * EITHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING ANY IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY
 * OR COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. TI DISCLAIMS
 * ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET POSSESSION, AND NON-INFRINGEMENT
 * OF ANY THIRD PARTY INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR
 * YOUR USE OF THE PROGRAM.
 * IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, CONSEQUENTIAL OR
 * INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY THEORY OF LIABILITY AND WHETHER OR
 * NOT TI HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY
 * OUT OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM.  EXCLUDED
 * DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF REMOVAL OR REINSTALLATION,
 * COMPUTER TIME, LABOR COSTS, LOSS OF GOODWILL, LOSS OF PROFITS, LOSS OF
 * SAVINGS, OR LOSS OF USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S
 * AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF YOUR USE OF THE
 * PROGRAM EXCEED FIVE HUNDRED DOLLARS (U.S.$500).
 * Unless otherwise stated, the Program written and copyrighted by Texas
 * Instruments is distributed as "freeware".  You may, only under TI's copyright
 * in the Program, use and modify the Program without any charge or restriction.
 * You may distribute to third parties, provided that you transfer a copy of this
 * license to the third party and the third party agrees to these terms by its
 * first use of the Program. You must reproduce the copyright notice and any
 * other legend of ownership on each copy or partial copy, of the Program.
 * You acknowledge and agree that the Program contains copyrighted material,
 * trade secrets and other TI proprietary information and is protected by
 * copyright laws, international copyright treaties, and trade secret laws, as
 * well as other intellectual property laws.  To protect TI's rights in the
 * Program, you agree not to decompile, reverse engineer, disassemble or
 * otherwise translate any object code versions of the Program to a
 * human-readable form.  You agree that in no event will you alter, remove or
 * destroy any copyright notice included in the Program.  TI reserves all rights
 * not specifically granted under this license. Except as specifically provided
 * herein, nothing in this agreement shall be construed as conferring by
 * implication, estoppel, or otherwise, upon you, any license or other right
 * under any TI patents, copyrights or trade secrets.
 * You may not use the Program in non-TI devices. */

//******************************************************************************
//
//  main.c
//  Cranking Simulator
//
//  Matthias Ulmann, Design Services - EMEA Power Management
//  Texas Instruments, Inc.
//  September 2013
//  Built with Code Composer Studio Core Edition Version: 5.5.0.00077
//
//	Device: MSP430F2232
//
//	Release 1.01
//
//******************************************************************************


#include "main.h"



// Global variables for scheduler
static volatile unsigned int counter_10ms = 0;	// Used for 10ms interrupt
static volatile unsigned int counter_100ms=0;	// Used for 100ms timing


// Global variables for program flow
volatile unsigned int status_prog = 1;			// Program 1, 2 or 3
volatile unsigned int status_enable = 0;		// Power supply enabled/disabled
volatile unsigned int status_continuous = 0;	// Continuous or single shot mode
volatile unsigned int status_pulse = 0;			// Information if a waveform has to be generated or not


// Global variable for AD-converter
volatile unsigned int poti_adc = 0;				// Set delay between pulses
												// 0..1023, 1 LSB equals 2ms additional delay






void main(void)
{
	WDTCTL = WDTPW + WDTHOLD;	// Stop watchdog

	init_clock();				// Initialization of clock system
	init_VID();					// Initialization of the VID interface
	init_switches();			// Initialization of the switches
	init_timerA();				// Initialization of timerA
	init_outputs();				// Initialization of the outputs
	init_adc();					// Initialization of the AD converter

	_EINT();					// Enable interrupts globally

	LED_SINGLE_on;				// Switch single mode LED on

	LED_PROG_1_on;				// Switch LED for program 1 on

	set_default_voltage();		// Switch to default output voltage


	 // enable conversion
	set_bit(ADC10CTL0, ENC);
	// start AD conversion
	set_bit(ADC10CTL0, ADC10SC);



	for(;;)
	{

		switch(status_pulse)
		{
		case 0:	// Stop generating a single pulse or continuous pulse
			// Nothing to do
			break;

		case 1:	// Generate a single pulse
			status_pulse = 2;				// A single pulse is generated at the moment
			LED_PULSE_on;					// Switch on LED

			switch(status_prog)
			{
			case 1:	generate_pulse_program_1();	break;
			case 2:	generate_pulse_program_2();	break;
			case 3:	generate_pulse_program_3();	break;
			}

			status_pulse = 0;				// Stop generating a single pulse or continuous pulses
			LED_PULSE_off;					// Switch off LED
			set_default_voltage();			// Switch back to default output voltage
			break;

		case 3:	// Generate continuous pulses
			status_pulse = 4;				// Continuous pulses are generated at the moment
			LED_PULSE_on;					// Switch on LED

			// Loop until status_pulse == 0 or user switched to single pulse mode
			while(status_pulse && status_continuous)
			{
				switch(status_prog)
				{
				case 1:	generate_pulse_program_1();	break;
				case 2:	generate_pulse_program_2();	break;
				case 3:	generate_pulse_program_3();	break;
				}
			}

			LED_PULSE_off;					// Switch off LED
			set_default_voltage();			// Switch back to default output voltage
			break;
		}

	}
}




// Executed every 10ms
#pragma vector = TIMERA0_VECTOR
__interrupt void TIMERA0_ISR(void)
{
	counter_10ms++;     	// Increase 10ms counter

	// Read and debounce inputs/switches
	read_debounce_inputs();



	if(counter_10ms > 9)	// 100ms bygone
	{
		counter_10ms = 0;	// Reset counter

		// Process inputs and set variables for program flow
		process_inputs();
	}
}
