/* THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR REPRESENTATIONS,
 * EITHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING ANY IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY
 * OR COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. TI DISCLAIMS
 * ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET POSSESSION, AND NON-INFRINGEMENT
 * OF ANY THIRD PARTY INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR
 * YOUR USE OF THE PROGRAM.
 * IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, CONSEQUENTIAL OR
 * INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY THEORY OF LIABILITY AND WHETHER OR
 * NOT TI HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY
 * OUT OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM.  EXCLUDED
 * DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF REMOVAL OR REINSTALLATION,
 * COMPUTER TIME, LABOR COSTS, LOSS OF GOODWILL, LOSS OF PROFITS, LOSS OF
 * SAVINGS, OR LOSS OF USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S
 * AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF YOUR USE OF THE
 * PROGRAM EXCEED FIVE HUNDRED DOLLARS (U.S.$500).
 * Unless otherwise stated, the Program written and copyrighted by Texas
 * Instruments is distributed as "freeware".  You may, only under TI's copyright
 * in the Program, use and modify the Program without any charge or restriction.
 * You may distribute to third parties, provided that you transfer a copy of this
 * license to the third party and the third party agrees to these terms by its
 * first use of the Program. You must reproduce the copyright notice and any
 * other legend of ownership on each copy or partial copy, of the Program.
 * You acknowledge and agree that the Program contains copyrighted material,
 * trade secrets and other TI proprietary information and is protected by
 * copyright laws, international copyright treaties, and trade secret laws, as
 * well as other intellectual property laws.  To protect TI's rights in the
 * Program, you agree not to decompile, reverse engineer, disassemble or
 * otherwise translate any object code versions of the Program to a
 * human-readable form.  You agree that in no event will you alter, remove or
 * destroy any copyright notice included in the Program.  TI reserves all rights
 * not specifically granted under this license. Except as specifically provided
 * herein, nothing in this agreement shall be construed as conferring by
 * implication, estoppel, or otherwise, upon you, any license or other right
 * under any TI patents, copyrights or trade secrets.
 * You may not use the Program in non-TI devices. */


//******************************************************************************
//
//  pulse.c
//  Definition of the cranking pulses
//
//  Matthias Ulmann, Design Services - EMEA Power Management
//  Texas Instruments, Inc.
//  September 2013
//  Built with Code Composer Studio Core Edition Version: 5.5.0.00077
//
//******************************************************************************


#include "main.h"



// Function to stop pulse generating during a pulse
unsigned int stop_pulse(void)
{
	if(status_pulse == 0)		// Check if user wants to stop
	{
		return 1;	// Exit if user tells
	}
	else return 0;
}



// Process pulse 1
void generate_pulse_program_1(void)
{
	/* Daimler-Chrysler Engine Cranking Test Pulse DC-10615
	 * ----------------------------------------------------
	 * 10ms @ 12.6V, trigger out active		NOT SPECIFIED
	 * Voltage decreases from 12.6V to 5.0V within 5ms
	 * 15ms @ 5.0V
	 * Voltage increases from 5.0V to 6.5V within 50ms
	 * 10s @ 6.5V
	 * Voltage increases from 6.5V to 12.6V within 100ms
	 * 100ms @ 12.6V						NOT SPECIFIED
	 */

	unsigned int vid_out = 207;		// Output of VID interface, 12.6V
	unsigned int counter = 0;		// Variable for delay


	// Set output to 12.6V ( = 207)
	VID_POUT = vid_out;

	// Set trigger out high
	TRIGGER_OUT_on;

	// Wait 10ms
	delay_ms(10);

	// Set trigger out low
	TRIGGER_OUT_off;

	// Decrease the voltage from 12.6 (207) to 5.0V (60) within 5ms
	while (vid_out > 60)
	{
		vid_out--;
		VID_POUT = vid_out;
	    delay_us(34);
	}

	// Stay at 5.0V (60) for 15ms
	delay_ms(15);

	if( stop_pulse() ) return;	// Exit if user tells

	// Increase the voltage from 5.0V (60) to 6.5V (89) within 50ms
	while (vid_out < 89)
	{
		VID_POUT = vid_out;
		vid_out++;
		delay_us(1700);
	}

	if( stop_pulse() ) return;	// Exit if user tells

	// Stay at 6.5V (89) for 10s
	counter = 0;
		while(counter < 100)
		{
			delay_ms(100);
			counter++;
			if( stop_pulse() ) return;	// Exit if user tells
		}

	// Increase the voltage from 6.5V (89) to 12.6V (207) within 100ms
	while (vid_out < 207)
	{
		VID_POUT = vid_out;
		vid_out++;
		delay_us(847);
	}

	if( stop_pulse() ) return;	// Exit if user tells

	// Stay at 12.6V (207) for 100ms
	delay_ms(100);

	if( stop_pulse() ) return;	// Exit if user tells

	// Add an additional delay depending on the potentiometer
	counter = 0;
	while(counter < poti_adc)
	{
		delay_ms(2);
		counter++;
		if( stop_pulse() ) return;	// Exit if user tells
	}
}



// Process pulse 2
void generate_pulse_program_2(void)
{
	/* Volkswagen Warm Start Test Pulse
	 * --------------------------------
	 * 10ms @ 11.0V, trigger out active
	 * Voltage decreases from 11.0V to 7.0V within < 1ms (750us)
	 * 15ms @ 7.0V
	 * Voltage increases from 7.0V to 8.0V within 70ms
	 * 240ms @ 8.0V
	 * Voltage increases from 8.0V to 9.0V within 70ms
	 * 600ms @ 9.0V
	 * Voltage increases from 9.0V to 11.0V within < 1ms (750us)
	 * 100ms @ 11.0V						NOT SPECIFIED
	 */

	unsigned int vid_out = 176;		// Output of VID interface, 11.0V
	unsigned int counter = 0;		// Variable for delay


	// Set output to 11.0V ( = 176)
	VID_POUT = vid_out;

	// Set trigger out high
	TRIGGER_OUT_on;

	// Wait 10ms
	delay_ms(10);

	// Set trigger out low
	TRIGGER_OUT_off;

	// Decrease the voltage from 11.0 (176) to 7.0V (99) within 750us
	while (vid_out > 99)
	{
		vid_out--;
		VID_POUT = vid_out;
	    delay_us(10);
	}

	// Stay at 7.0V (99) for 15ms
	delay_ms(15);

	if( stop_pulse() ) return;	// Exit if user tells

	// Increase the voltage from 7.0V (99) to 8.0V (118) within 70ms
	while (vid_out < 118)
	{
		VID_POUT = vid_out;
		vid_out++;
		delay_us(3684);
	}

	if( stop_pulse() ) return;	// Exit if user tells

	// Stay at 8.0V (118) for 240ms
	delay_ms(120);
	if( stop_pulse() ) return;	// Exit if user tells
	delay_ms(120);

	// Increase the voltage from 8.0V (118) to 9.0V (137) within 70ms
	while (vid_out < 137)
	{
		VID_POUT = vid_out;
		vid_out++;
		delay_us(3684);
	}

	if( stop_pulse() ) return;	// Exit if user tells

	// Stay at 9.0V (137) for 600ms
	counter = 0;
	while(counter < 6)
	{
		delay_ms(100);
		counter++;
		if( stop_pulse() ) return;	// Exit if user tells
	}

	// Increase the voltage from 9.0V (137) to 11.0V (176) within 750us
	while (vid_out < 176)
	{
		VID_POUT = vid_out;
		vid_out++;
		delay_us(20);
	}

	// Stay at 11.0V (176) for 100ms
	delay_ms(100);

	if( stop_pulse() ) return;	// Exit if user tells

	// Add an additional delay depending on the potentiometer
	counter = 0;
	while(counter < poti_adc)
	{
		delay_ms(2);
		counter++;
		if( stop_pulse() ) return;	// Exit if user tells
	}
}



// Process pulse 3
void generate_pulse_program_3(void)
{
	/* Volkswagen Cold Start Test Pulse
	 * --------------------------------
	 * 10ms @ 11.0V, trigger out active
	 * Voltage decreases from 11.0V to 3.2V within < 1ms (750us)
	 * 19ms @ 3.2V
	 * Voltage increases from 3.2V to 5.0V within < 1ms (750us)
	 * 329ms @ 5.0V
	 * Voltage increases from 5.0V to 6.0V within 50ms
	 * Sine waveform, Umin (6V)/Umax(8V) with 2 Hz, 20 periods
	 * Voltage increases from 6.0V to 11.0V within 100ms
	 * 100ms @ 11.0V						NOT SPECIFIED
	 */

	unsigned int vid_out = 176;		// Output of VID interface, 11.0V
	unsigned int counter = 0;		// Variable for delay


	// Set output to 11.0V ( = 176)
	VID_POUT = vid_out;

	// Set trigger out high
	TRIGGER_OUT_on;

	// Wait 10ms
	delay_ms(10);

	// Set trigger out low
	TRIGGER_OUT_off;

	// Decrease the voltage from 11.0 (176) to 3.2V (25) within 750us
	while (vid_out > 25)
	{
		vid_out--;
		VID_POUT = vid_out;
	    delay_us(5);
	}

	// Stay at 3.2V (25) for 19ms
	delay_ms(19);

	if( stop_pulse() ) return;	// Exit if user tells

	// Increase the voltage from 3.2V (25) to 5.0V (60) within 750us
	while (vid_out < 60)
	{
		VID_POUT = vid_out;
		vid_out++;
		delay_us(21);
	}

	if( stop_pulse() ) return;	// Exit if user tells

	// Stay at 5.0V (60) for 329ms
	delay_ms(110);
	if( stop_pulse() ) return;	// Exit if user tells
	delay_ms(110);
	if( stop_pulse() ) return;	// Exit if user tells
	delay_ms(110);

	// Increase the voltage from 5.0V (60) to 6.0V (79) within 50ms
	while (vid_out < 79)
	{
		VID_POUT = vid_out;
		vid_out++;
		delay_us(2632);
	}

	if( stop_pulse() ) return;	// Exit if user tells

	// Transition to sine
	VID_POUT = 80;	delay_ms(10);
	VID_POUT = 81;	delay_ms(10);
	VID_POUT = 83;	delay_ms(10);
	VID_POUT = 84;	delay_ms(10);
	VID_POUT = 86;	delay_ms(10);
	VID_POUT = 88;	delay_ms(10);
	VID_POUT = 90;	delay_ms(10);
	VID_POUT = 92;	delay_ms(10);
	VID_POUT = 94;	delay_ms(10);
	VID_POUT = 97;	delay_ms(10);
	VID_POUT = 98;	delay_ms(10);

	if( stop_pulse() ) return;	// Exit if user tells

	// Process sine waveform
	// Umin = 6V (79), Umax = 8V (118), f = 2 Hz,  20 periods (500ms)
	counter = 0;
	while(counter < 20)
	{
		VID_POUT = 99;	delay_ms(10);
		VID_POUT = 101;	delay_ms(10);
		VID_POUT = 104;	delay_ms(10);
		VID_POUT = 106;	delay_ms(10);
		VID_POUT = 108;	delay_ms(10);
		VID_POUT = 110;	delay_ms(10);
		VID_POUT = 112;	delay_ms(10);
		VID_POUT = 114;	delay_ms(10);
		VID_POUT = 115;	delay_ms(10);
		VID_POUT = 117;	delay_ms(10);
		if( stop_pulse() ) return;	// Exit if user tells
		VID_POUT = 118;	delay_ms(10);
		VID_POUT = 118;	delay_ms(10);
		VID_POUT = 118;	delay_ms(10);
		VID_POUT = 118;	delay_ms(10);
		VID_POUT = 118;	delay_ms(10);
		VID_POUT = 118;	delay_ms(10);
		VID_POUT = 117;	delay_ms(10);
		VID_POUT = 115;	delay_ms(10);
		VID_POUT = 114;	delay_ms(10);
		VID_POUT = 112;	delay_ms(10);
		if( stop_pulse() ) return;	// Exit if user tells
		VID_POUT = 110;	delay_ms(10);
		VID_POUT = 108;	delay_ms(10);
		VID_POUT = 106;	delay_ms(10);
		VID_POUT = 104;	delay_ms(10);
		VID_POUT = 101;	delay_ms(10);
		VID_POUT = 99;	delay_ms(10);
		VID_POUT = 97;	delay_ms(10);
		VID_POUT = 94;	delay_ms(10);
		VID_POUT = 92;	delay_ms(10);
		VID_POUT = 90;	delay_ms(10);
		if( stop_pulse() ) return;	// Exit if user tells
		VID_POUT = 88;	delay_ms(10);
		VID_POUT = 86;	delay_ms(10);
		VID_POUT = 84;	delay_ms(10);
		VID_POUT = 83;	delay_ms(10);
		VID_POUT = 81;	delay_ms(10);
		VID_POUT = 80;	delay_ms(10);
		VID_POUT = 80;	delay_ms(10);
		VID_POUT = 80;	delay_ms(10);
		VID_POUT = 80;	delay_ms(10);
		VID_POUT = 80;	delay_ms(10);
		if( stop_pulse() ) return;	// Exit if user tells
		VID_POUT = 80;	delay_ms(10);
		VID_POUT = 81;	delay_ms(10);
		VID_POUT = 83;	delay_ms(10);
		VID_POUT = 84;	delay_ms(10);
		VID_POUT = 86;	delay_ms(10);
		VID_POUT = 88;	delay_ms(10);
		VID_POUT = 90;	delay_ms(10);
		VID_POUT = 92;	delay_ms(10);
		VID_POUT = 94;	delay_ms(10);
		VID_POUT = 97;	delay_ms(10);
		VID_POUT = 99;
		if( stop_pulse() ) return;	// Exit if user tells

		counter++;
	}

	vid_out = 99;	// Set to 99 go get a smooth transition between the sine and the slope

	// Increase the voltage from 6.0V (79) to 11.0V (176) within 100ms
	while (vid_out < 176)
	{
		VID_POUT = vid_out;
		vid_out++;
		delay_us(1031);
	}

	// Stay at 11.0V (176) for 100ms
	delay_ms(100);

	if( stop_pulse() ) return;	// Exit if user tells

	// Add an additional delay depending on the potentiometer
	counter = 0;
	while(counter < poti_adc)
	{
		delay_ms(2);
		counter++;
		if( stop_pulse() ) return;	// Exit if user tells
	}
}



// Set default output voltage depending on mode
void set_default_voltage(void)
{
	switch(status_prog)
	{
	case 1:
		VID_POUT = vid_out_max_1;
		break;

	case 2:
		VID_POUT = vid_out_max_2;
		break;

	case 3:
		VID_POUT = vid_out_max_3;
		break;
	}
}



// Process pulse for test mode 1 (rectangular)
void generate_pulse_test_mode_1(void)
{
	while(1)
	{
		// Set output to 2.0V (0)
		VID_POUT = 0;

		// Stay at 2.0V (0) for 10ms
		delay_ms(10);

		// Set output to 15.0V (254)
		VID_POUT = 254;

		// Stay at 15.0V (254) for 10ms
		delay_ms(10);

		if( stop_pulse() ) return;	// Exit if user tells
	}
}



// Process pulse for test mode 2 (sawtooth)
void generate_pulse_test_mode_2(void)
{
	unsigned int vid_out = 0;		// Output of VID interface, 0V


	while(1)
	{
		// Set output to 2.0V (0)
		VID_POUT = vid_out;

		// Increase the voltage from 2.0V (0) to 15.0V (254) within 10ms
		while (vid_out < 254)
		{
			VID_POUT = vid_out;
			vid_out++;
			delay_us(39);
		}

		vid_out = 0;

		if( stop_pulse() ) return;	// Exit if user tells
	}
}



// Process pulse for test mode 3 (sine)
void generate_pulse_test_mode_3(void)
{
	VID_POUT = 0;

	while(1)
	{
		// Process sine waveform
		// Umin = 2V (0), Umax = 15V (254), f = 50 Hz
		VID_POUT = 127;	delay_us(100);
		VID_POUT = 135;	delay_us(100);
		VID_POUT = 143;	delay_us(100);
		VID_POUT = 151;	delay_us(100);
		VID_POUT = 159;	delay_us(100);
		VID_POUT = 166;	delay_us(100);
		VID_POUT = 174;	delay_us(100);
		VID_POUT = 181;	delay_us(100);
		VID_POUT = 188;	delay_us(100);
		VID_POUT = 195;	delay_us(100);
		VID_POUT = 202;	delay_us(100);
		VID_POUT = 208;	delay_us(100);
		VID_POUT = 214;	delay_us(100);
		VID_POUT = 220;	delay_us(100);
		VID_POUT = 225;	delay_us(100);
		VID_POUT = 230;	delay_us(100);
		VID_POUT = 234;	delay_us(100);
		VID_POUT = 238;	delay_us(100);
		VID_POUT = 242;	delay_us(100);
		VID_POUT = 245;	delay_us(100);
		VID_POUT = 248;	delay_us(100);
		VID_POUT = 250;	delay_us(100);
		VID_POUT = 252;	delay_us(100);
		VID_POUT = 253;	delay_us(100);
		VID_POUT = 254;	delay_us(100);
		VID_POUT = 254;	delay_us(100);
		VID_POUT = 254;	delay_us(100);
		VID_POUT = 253;	delay_us(100);
		VID_POUT = 252;	delay_us(100);
		VID_POUT = 250;	delay_us(100);
		VID_POUT = 248;	delay_us(100);
		VID_POUT = 245;	delay_us(100);
		VID_POUT = 242;	delay_us(100);
		VID_POUT = 238;	delay_us(100);
		VID_POUT = 234;	delay_us(100);
		VID_POUT = 230;	delay_us(100);
		VID_POUT = 225;	delay_us(100);
		VID_POUT = 220;	delay_us(100);
		VID_POUT = 214;	delay_us(100);
		VID_POUT = 208;	delay_us(100);
		VID_POUT = 195;	delay_us(100);
		VID_POUT = 188;	delay_us(100);
		VID_POUT = 181;	delay_us(100);
		VID_POUT = 174;	delay_us(100);
		VID_POUT = 166;	delay_us(100);
		VID_POUT = 159;	delay_us(100);
		VID_POUT = 151;	delay_us(100);
		VID_POUT = 143;	delay_us(100);
		VID_POUT = 135;	delay_us(100);
		VID_POUT = 127;	delay_us(100);
		VID_POUT = 119;	delay_us(100);
		VID_POUT = 111;	delay_us(100);
		VID_POUT = 103;	delay_us(100);
		VID_POUT = 95;	delay_us(100);
		VID_POUT = 88;	delay_us(100);
		VID_POUT = 80;	delay_us(100);
		VID_POUT = 73;	delay_us(100);
		VID_POUT = 66;	delay_us(100);
		VID_POUT = 59;	delay_us(100);
		VID_POUT = 52;	delay_us(100);
		VID_POUT = 46;	delay_us(100);
		VID_POUT = 40;	delay_us(100);
		VID_POUT = 34;	delay_us(100);
		VID_POUT = 29;	delay_us(100);
		VID_POUT = 24;	delay_us(100);
		VID_POUT = 20;	delay_us(100);
		VID_POUT = 16;	delay_us(100);
		VID_POUT = 12;	delay_us(100);
		VID_POUT = 9;	delay_us(100);
		VID_POUT = 6;	delay_us(100);
		VID_POUT = 4;	delay_us(100);
		VID_POUT = 2;	delay_us(100);
		VID_POUT = 1;	delay_us(100);
		VID_POUT = 0;	delay_us(100);
		VID_POUT = 0;	delay_us(100);
		VID_POUT = 0;	delay_us(100);
		VID_POUT = 1;	delay_us(100);
		VID_POUT = 2;	delay_us(100);
		VID_POUT = 4;	delay_us(100);
		VID_POUT = 6;	delay_us(100);
		VID_POUT = 9;	delay_us(100);
		VID_POUT = 12;	delay_us(100);
		VID_POUT = 16;	delay_us(100);
		VID_POUT = 20;	delay_us(100);
		VID_POUT = 24;	delay_us(100);
		VID_POUT = 29;	delay_us(100);
		VID_POUT = 34;	delay_us(100);
		VID_POUT = 40;	delay_us(100);
		VID_POUT = 46;	delay_us(100);
		VID_POUT = 52;	delay_us(100);
		VID_POUT = 59;	delay_us(100);
		VID_POUT = 66;	delay_us(100);
		VID_POUT = 73;	delay_us(100);
		VID_POUT = 80;	delay_us(100);
		VID_POUT = 88;	delay_us(100);
		VID_POUT = 95;	delay_us(100);
		VID_POUT = 103;	delay_us(100);
		VID_POUT = 111;	delay_us(100);
		VID_POUT = 119;	delay_us(100);
		VID_POUT = 127;	delay_us(100);

		if( stop_pulse() ) return;	// Exit if user tells
	}
}
